local game = {}

function game:init(music,rate,speed)
    self.music = music
    self.music:play()
    self.lane = {
        {},{},{},{}
    } --four
    self.speed = speed or 0.5
    
    self.music.onNote = function(m,key) 
        self:addNote(key)
    end
    self.down = {
        {on = false, timer = 0},
        {on = false, timer = 0},
        {on = false, timer = 0},
        {on = false, timer = 0},
    }
    self.score = {
        miss = 0,
        bad = 0,
        good = 0,
        cool = 0
    }
    self.rate = rate
    self.keyTimer = 0
    
end

function game:newBlock(k)
    return {
        time = self.speed,
        key = k
        }
end

function game:addNote(k)
    --if love.math.random()<.7 then return end
    if self.keyTimer>0 then return end
    self.keyTimer = 1/self.rate
    local mod = k%4+1
    table.insert(self.lane[mod],self:newBlock(k))
end

function game:readKey(key)
    if key == "d" then
        self.down[1].timer = 0.3
        self:checkLane(1)
    elseif key == "f" then
        self.down[2].timer = 0.3
        self:checkLane(2)
    elseif key == "j" then
        self.down[3].timer = 0.3
        self:checkLane(3)
    elseif key == "k" then
        self.down[4].timer = 0.3
        self:checkLane(4)
    end
end

function game:checkLane(m)
    local last = self.lane[m][1]
    if not last then return end
    if last.time>0.5 then return end
    if last.time>0.3 then self.score.bad = self.score.bad + 1
    elseif last.time>0.1 then self.score.good = self.score.good + 1
    elseif last.time>-0.1 then self.score.cool = self.score.cool + 1 
    elseif last.time>-0.3 then self.score.good = self.score.good + 1
    elseif last.time>-0.5 then self.score.bad = self.score.bad + 1 end
    table.remove(self.lane[m],1)
end

function game:update(dt)
    self.keyTimer = self.keyTimer - dt
    self.music:update(dt)
    for i,lane in ipairs(self.lane) do
        for ii =  #lane,1,-1 do
            local block = lane[ii]
            block.time = block.time - dt
            if block.time< - 0.5 then
                self.score.miss = self.score.miss + 1
                table.remove(lane,ii)
            end
        end
    end
    for i = 1,4 do
       self.down[i].timer = self.down[i].timer - dt
    end
end

function game:draw()
    for i,lane in ipairs(self.lane) do
        for ii =  #lane,1,-1 do
            local block = lane[ii]
            love.graphics.setColor(0.2,0.3,0.9,1)
            love.graphics.rectangle("fill",i*150,500 - 500*block.time/self.speed,100,30)
        end
    end
    love.graphics.setColor(1,0.4,0.4,1)
    love.graphics.line(0,500,800,500)
    
    for i = 1,4 do
        if self.down[i].timer >0 then
            love.graphics.setColor(0.5,0.8,0.2,0.3)
            love.graphics.rectangle("fill",i*150,0,100,500)
        end
    end
    local text = string.format("cool: %d, good: %d, bad: %d, miss: %d",
        self.score.cool,self.score.good,self.score.bad,self.score.miss)
    love.graphics.printf(text,0,200,800,"center")
end

return game