local midi = {}
local parser = require "midi-parser"

--[[
BPM = 60,000,000/MicroTempo 
MicrosPerPPQN = MicroTempo/TimeBase 
MicrosPerMIDIClock = MicroTempo/24 
PPQNPerMIDIClock = TimeBase/24 
MicrosPerSubFrame = 1000000 * Frames * SubFrames 
SubFramesPerQuarterNote = MicroTempo/(Frames * SubFrames) 
SubFramesPerPPQN = SubFramesPerQuarterNote/TimeBase 
MicrosPerPPQN = SubFramesPerPPQN * Frames * SubFrames
]]

function midi:newPlayer(midiPlayer,path)
    local player ={
        source = parser(path),
        state = "stopped",
        loopType = "loop",
        timer = 0,
        heads = {  
        },
        on = {},
        midiPlayer = midiPlayer
    }
    player.timebase = player.source.timebase
    for i,track in ipairs(player.source.tracks) do
        player.heads[i] = {
            timer = 0,
            index = 1
        }
    end
    
    function player:printData()
        print(table.save(self.source))
    end
    function player:play()
        player.state = "playing"
    end
    
    function player:onNote(note)
        --print(123)
    end
    
    function player:readMessage(i,msg)
        if not msg then return end
        if msg.meta == "Set Tempo" then
            self.tempo = msg.tempo --每个四分音符多少微秒
            self.bpm = 60000000/self.tempo --每分钟多少节拍
            self.bps = self.bpm/60 
            self.PPQN = self.tempo/self.timebase
            self.pulseLength = 60/(self.PPQN*self.bpm)
        elseif msg.meta == "Time Signature" then
            self.numerator = msg.signature[1] --以几分音符为一拍
            self.denominator = 2^msg.signature[2] --每小节几拍
            self.ticksPerClick = msg.signature[3] --节拍器速率，几个midi时钟响一次
            self.multiplier = msg.signature[4] --速度倍率，8 对应四分音符的正常倍率（8/32 nds）
            --self.pulseLength = 60/(self.PPQN*self.bpm) --每个pulse的声音时长
            print(self.PPQN , self.pulseLength, self.bpm,self.bps)
        elseif msg.meta == "Track Name" then
            self.name = msg.text
        elseif msg.type == "on" then
            if self.midiPlayer then
                self.on[i] = self.on[i] or {}
                self.on[i][msg.number] = self.midiPlayer:playNumber(
                    msg.number,
                    false,
                    msg.velocity/500
                )
            end
            self:onNote(msg.number)
            return msg.time
        elseif msg.type == "off" then
            if self.midiPlayer then
                self.on[i][msg.number]:stop()
                self.on[i][msg.number]= nil
            end
            return msg.time
        end
    end
    
    
    function player:go(dt)
        for i,track in ipairs(self.source.tracks) do
            self.heads[i].timer = self.heads[i].timer - dt
            if not self.heads[i].over and self.heads[i].timer <= 0 then
                while true do
                    local index = self.heads[i].index
                    local msg = track.messages[index]
                    if not msg then self.heads[i].over = true ;break end
                    --print(i,index,msg.time)
                    self:readMessage(i,msg)
                    local nextMsg= track.messages[index + 1]
                    if not nextMsg then self.heads[i].over = true ;break end
                    if nextMsg.time ~= 0 then
                        self.heads[i].index = index + 1
                        self.heads[i].timer = self.heads[i].timer + nextMsg.time*self.PPQN/1000000
                        break
                    else
                        self.heads[i].index = index + 1
                    end
                end
            end
        end
    end
    
    
    function player:update(dt)
        if self.state == "playing" then
           self:go(dt) 
        end
    end
    
    return player
end

function midi:new(midiPlayer,path)
    local player = self:newPlayer(midiPlayer,path)
    return player
end

    
    
return midi