local player = {}
player.pool = {}
player.height =2
player.show = {}
player.buffer = {}
local noteName = {
  [1] = 1,
  [2] = 3,
  [3] = 5,
  [4] = 6,
  [5] = 8,
  [6] = 10,
  [7] = 12,
  c = 1,
  d = 3,
  e = 5,
  f = 6,
  g = 8,
  a = 10,
  b = 12
}

local toName = {
 [1] = 1,
 [2] = 1.5,
 [3] = 2,
 [4] = 2.5,
 [5] = 3,
 [6] = 4,
 [7] = 4.5,
 [8] = 5,
 [9] = 5.5,
 [10] = 6,
 [11] = 6.5,
 [12] = 7
}


function player:addShow(note)
    table.insert(self.show,{
        x = 100+(note-20)*30,
        y = 0,
        h = math.floor((note-30)/12),
        n = (note-30)%12 + 1,
    })
end

function player:setMainNote(note)
    self.mainNote = note
end

function player:getSource(pitch)
    if self.buffer[pitch] then
        for i = 1,10 do
            if self.buffer[pitch][i] and (not self.buffer[pitch][i]:isPlaying()) then
               return  self.buffer[pitch][i]
            end
        end
    else
        local source = love.audio.newSource(self.mainNote)
        self:setBuffer(pitch,source)
        return source
    end
end

function player:setBuffer(pitch,source)
    self.buffer[pitch] = self.buffer[pitch] or {}
    for i = 1,10 do
       if not self.buffer[pitch][i] then self.buffer[pitch][i] = source end
    end
end

function player:playNumber(num,length,power)
    num = num - 36
    local h = 2^(num/12-4)
    local source = self:getSource(h)
    --name = type(name) == "string" and noteName[name] or name
    source:setPitch(h)
    source:setVolume(power)
    source:setLooping(true)
    source:play()
    table.insert(player.pool,{source,length or 3,num/12*5})
    return source
end

function player:playNote(height,name,length,power)
    height = height - 5
    local source = love.audio.newSource(self.mainNote)
    --print(source:getDuration())
    local v = height*12+noteName[name]-1
    local h = 2^(v/12)
    source:setVolume(power or 1)
    source:setPitch(h)
    source:setLooping(false)
    source:play()
    table.insert(player.pool,{source,length,v})
    return source
end

function player:clear()
    for i,source in ipairs(self.pool) do
        source[1]:stop()
    end
    self.pool = {}
end

function player:keypressed(key)
    local function down(k) 
        return k == key
    end
    if down("kp0") then
        player:clear()
    elseif down("kp1") or down("z") then
        player:playNote(player.height,"c",5)
    elseif down("kp2") or down("x") then
        player:playNote(player.height,"d",5)
    elseif down("kp3") or down("c") then
        player:playNote(player.height,"e",5)
    elseif down("kp4") or down("a") then
        player:playNote(player.height,"f",5)
    elseif down("kp5") or down("s") then
        player:playNote(player.height,"g",5)
    elseif down("kp6") or down("d") then
        player:playNote(player.height,"a",5)
    elseif down("kp7") or down("q") then
        player:playNote(player.height,"b",5)
    end
end

function player.keyboard()
   local down = love.keyboard.isDown
    if down("kp0") then
        player:clear()
    elseif down("1") then
        player.height = 1
    elseif down("2") then
        player.height = 2
    elseif down("3") then
        player.height = 3
    elseif down("4") then 
        player.height = 4
    end
end

function player:update(dt)
    self:keyboard()
    for i = #self.pool,1,-1 do
        local note = self.pool[i]
        if note[2] then
            note[2] = note[2] - dt
        end
        
        if note[2] and note[2]<0 then
            note[1]:stop()
            table.remove(self.pool,i)
        end
    end
    
    for i, v in ipairs(self.show) do
        v.y = v.y + 200*dt
    end
    --print(collectgarbage("count"))
    if collectgarbage("count")>1000 then
        collectgarbage("collect")
        collectgarbage("stop")
    end
end

function player:draw()
    
    if player.pool[1] then
        for i,v in ipairs(self.pool) do
            local v = player.pool[i][3]
            local r, g, b = math.HSVtoRGB(360*v/49,1,255)
            local a = 255/#player.pool
            love.graphics.setColor(r,g,b,255)
            love.graphics.circle("fill",300,100,30)
        end
    end 
    
    for i,v in ipairs(self.show) do
        love.graphics.setColor(50,150,250)
        love.graphics.rectangle("fill",v.x,v.y,30,50)
        love.graphics.setColor(250,20,50)
        love.graphics.print(toName[v.n],v.x+8,v.y+10)
    end
end

return player