local timbre = {
    data = {},
    x = 0,
    y = 0,
    w = 1000,
    h = 200,
    rate = 100,
    freq = 441,
    length = 100, --sec
}

local shader = love.graphics.newShader("audioShader.glsl")
function timbre:init()
    self.canvas = love.graphics.newCanvas(self.w,self.h)
    self.audio = love.graphics.newCanvas(self.rate*self.length,1)
    for i = 1,self.rate*self.length do
        self.data[i] = 0
    end
    self:drawData()
    self:getMainNote()
end

function timbre:drawData()
    love.graphics.setCanvas(self.audio)
        love.graphics.clear(0,0,0,0)
        for i ,d in ipairs(self.data) do
            love.graphics.setShader(shader)
            love.graphics.setColor(1,1,1,1)
            love.graphics.points(i,1)
            love.graphics.setShader()
        end
    love.graphics.setCanvas()
    local data = self.audio:newImageData()
    for i ,d in ipairs(self.data) do
       self.data[i] = data:getPixel(i-1,0)-0.5
    end
    
    love.graphics.setCanvas(self.canvas)
        love.graphics.clear(0.3,0.3,0.5,1)
        for i ,d in ipairs(self.data) do
            love.graphics.line(
                i*self.w/self.rate/self.length,-100*d+100,
                (i-1)*self.w/self.rate/self.length,-100*(self.data[i-1] or 0)+100
            )
        end
    love.graphics.setCanvas()
    
    
end

function timbre:checkHover()
    local mx,my = love.mouse.getPosition()
    if mx<self.x or mx>self.x+self.w then return end
    if my<self.y or my>self.y+self.h then return end
    return true
end

function timbre:update()
    if timbre:checkHover() and love.mouse.isDown(1) then
        local mx,my = love.mouse.getPosition()
        local x = math.ceil(mx/2)
        local y = 1 - my/100
        self.data[x] = y
        self:drawData()
    end
    
end

function timbre:getMainNote()
    local mainNote = love.sound.newSoundData(self.rate*self.freq*self.length,44100,16,1)
    for i = 0, #self.data-1 do
        mainNote:setSample(i,self.data[i+1]) 
    end
    return mainNote
end

function timbre:draw()
    love.graphics.setColor(255,255,255)
    love.graphics.draw(self.canvas,self.x,self.y)
end

return timbre